clear all
clc
addpath(genpath('utils'));
%% best parameter
% ||Ps - D1*Zs|| + lambda * ||As - D2*Zs|| + alpha * (||Pu - D1*Zu|| + lambda * ||Au - D2*Zu||) + beta * ||X- Ps*H||

%% prepare data
path = '../code/datasets_resnet';
dataset = 'AWA1';
[ train_feature, test_feature , train_label, test_label, trainClassLabels, testClassLabels, semantic,name] = prepare_data_resnet(path,dataset);

pars.dimFeature=size(train_feature,1);
pars.dimSemantic = size(semantic,2);
pars.numTrainClass = length(trainClassLabels);
%% get the initial prototype of seen classes in the image space
Ps_i = center(train_feature, train_label, trainClassLabels);
As = semantic(trainClassLabels,:)';
Au = semantic(testClassLabels,:)';
X = train_feature;
H = get_H(trainClassLabels, train_label);
SIM = semantic*semantic';


pars.gamma =1;

%% initialize 
Zu_i = SIM(trainClassLabels,testClassLabels);
D2_i = learn_basis(Au,Zu_i, pars.gamma);
Zs_i = learn_coefficients_noise(D2_i,As,pars.gamma);
D1_i = learn_basis(Ps_i,Zs_i, pars.gamma);
Pu_i = D1_i*Zu_i;
[ ac_v1, ac_a1, ac_s1, ac_va1, ac_as1, ac_vs1, ac_vas1 ] = evaluate_all(test_feature,Pu_i,Zu_i,test_label,Au,testClassLabels, D1_i, D2_i, pars.gamma);

%% parameter settings

pars.iter=100;

% % apy param
% param_lambda = 1 ;
% param_alpha = 0.01;
% param_beta = 0.001 ;
% pars.iter=20;

% awa param
param_lambda = 1 ;
param_alpha = 1;
param_beta = 0.1 ;
pars.iter=50;

% % cub param
% param_lambda = 1 ;
% param_alpha = 0.1;
% param_beta = 0.1 ;
% pars.iter=100;

% % sun param
% param_lambda = 10 ;
% param_alpha = 0.001;
% param_beta = 0.01 ;
% pars.iter=90;

%% optimization
result.TACC_v = [];
result.TACC_a= [];
result.TACC_s= [];
result.TACC_va = [];
result.TACC_as= [];
result.TACC_vs= [];
result.TACC_vas = [];


lambda= param_lambda;
alpha= param_alpha;
beta= param_beta;
Zs = Zs_i;
D1 = D1_i;
D2= D2_i ;
Zu = Zu_i;
Pu = Pu_i;
ACC_v = [];
ACC_a= [];
ACC_s= [];
ACC_va = [];
ACC_as= [];
ACC_vs= [];
ACC_vas = [];
ACC_v = [ACC_v ac_v1];
ACC_a= [ACC_a ac_a1];
ACC_s= [ACC_s ac_s1];
ACC_va = [ACC_va ac_va1];
ACC_as= [ACC_as ac_as1];
ACC_vs= [ACC_vs ac_vs1];
ACC_vas = [ACC_vas ac_vas1];
for m = 1:pars.iter
 	% update Ps
    m
	M = [D1*Zs beta*X];
	N = [eye(size(H,1)) beta*H];
	Ps = learn_basis(M,N, pars.gamma);
 	% update Zs
	P = [Ps ; lambda*As];
 	D = [D1 ; lambda*D2];
  	Zs = learn_coefficients_noise(D,P,pars.gamma);
	% update D1
	P = [Ps alpha*Pu];
 	Z = [Zs alpha*Zu];
 	D1 = learn_basis(P,Z, pars.gamma);
                
	% update D2
 	A = [As alpha*Au];
	Z = [Zs alpha*Zu];
	D2 = learn_basis(A,Z, pars.gamma);
                
	% update Zu
	P = [Pu ; lambda*Au];
	D = [D1 ; lambda*D2];
	Zu = learn_coefficients_noise(D,P,pars.gamma);
    
	% update Pu
 	Pu = D1*Zu;
 	[ ac_v, ac_a, ac_s, ac_va, ac_as, ac_vs, ac_vas] = evaluate_all( test_feature, Pu, Zu, test_label, Au,testClassLabels, D1, D2, pars.gamma);
    ACC_v = [ACC_v ac_v];
    ACC_a= [ACC_a ac_a];
    ACC_s= [ACC_s ac_s];
    ACC_va = [ACC_va ac_va];
    ACC_as= [ACC_as ac_as];
    ACC_vs= [ACC_vs ac_vs];
    ACC_vas = [ACC_vas ac_vas];
 end
result.TACC_V = ACC_v;
result.TACC_a= ACC_a;
result.TACC_s= ACC_s;
result.TACC_va = ACC_va;
result.TACC_as= ACC_as;
result.TACC_vs= ACC_vs;
result.TACC_vas = ACC_vas;
